@@ -55,7 +55,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: CoyoteAdapter.java 1240861 2012-02-05 23:30:20Z markt $
+ * @version $Id: CoyoteAdapter.java 1409036 2012-11-13 23:55:26Z markt $
  */
 public class CoyoteAdapter implements Adapter {
 
@@ -283,14 +283,14 @@ public class CoyoteAdapter implements Adapter {
                     ctxt.fireRequestDestroyEvent(request);
                 }
                 // Lift any suspension (e.g. if sendError() was used by an async
-                // request
+                // request) to allow the response to be written to the client
                 response.setSuspended(false);
             }
 
             if (status==SocketStatus.TIMEOUT) {
                 success = true;
                 if (!asyncConImpl.timeout()) {
-                    asyncConImpl.setErrorState(null);
+                    asyncConImpl.setErrorState(null, false);
                 }
             }
             if (request.isAsyncDispatching()) {
@@ -299,7 +299,7 @@ public class CoyoteAdapter implements Adapter {
                 Throwable t = (Throwable) request.getAttribute(
                         RequestDispatcher.ERROR_EXCEPTION);
                 if (t != null) {
-                    asyncConImpl.setErrorState(t);
+                    asyncConImpl.setErrorState(t, true);
                 }
             }
 
@@ -719,7 +719,19 @@ public class CoyoteAdapter implements Adapter {
                     }
                 }
             }
-
+            if (!mapRequired && request.getContext().getPaused()) {
+                // Found a matching context but it is paused. Mapping data will
+                // be wrong since some Wrappers may not be registered at this
+                // point.
+                try {
+                    Thread.sleep(1000);
+                } catch (InterruptedException e) {
+                    // Should never happen
+                }
+                // Reset mapping
+                request.getMappingData().recycle();
+                mapRequired = true;
+            }
         }
 
         // Possible redirect
