@@ -5,34 +5,30 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
-
+import java.beans.Introspector;
 import java.io.IOException;
 import java.io.InputStream;
-import java.lang.ref.WeakReference;
-import java.lang.reflect.AccessibleObject;
+import java.lang.annotation.Annotation;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.lang.reflect.Modifier;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
@@ -53,26 +49,39 @@ import javax.xml.ws.WebServiceRef;
 import org.apache.catalina.ContainerServlet;
 import org.apache.catalina.Globals;
 import org.apache.catalina.security.SecurityUtil;
+import org.apache.catalina.util.Introspection;
 import org.apache.tomcat.InstanceManager;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * @version $Id: DefaultInstanceManager.java 1176588 2011-09-27 20:13:32Z markt $
+ * @version $Id: DefaultInstanceManager.java 1431300 2013-01-10 11:47:49Z markt $
  */
 public class DefaultInstanceManager implements InstanceManager {
 
+    // Used when there are no annotations in a class
+    private static final AnnotationCacheEntry[] ANNOTATIONS_EMPTY
+        = new AnnotationCacheEntry[0];
+
+    /**
+     * The string manager for this package.
+     */
+    protected static final StringManager sm =
+        StringManager.getManager(Constants.Package);
+
     private final Context context;
     private final Map<String, Map<String, String>> injectionMap;
     protected final ClassLoader classLoader;
     protected final ClassLoader containerClassLoader;
     protected boolean privileged;
     protected boolean ignoreAnnotations;
-    private Properties restrictedFilters = new Properties();
-    private Properties restrictedListeners = new Properties();
-    private Properties restrictedServlets = new Properties();
-    private final Map<Class<?>,WeakReference<List<AnnotationCacheEntry>>> annotationCache =
-        new WeakHashMap<Class<?>, WeakReference<List<AnnotationCacheEntry>>>();
+    private final Properties restrictedFilters = new Properties();
+    private final Properties restrictedListeners = new Properties();
+    private final Properties restrictedServlets = new Properties();
+    private final Map<Class<?>, AnnotationCacheEntry[]> annotationCache =
+        new WeakHashMap<Class<?>, AnnotationCacheEntry[]>();
+    private final Map<String, String> postConstructMethods;
+    private final Map<String, String> preDestroyMethods;
 
     public DefaultInstanceManager(Context context, Map<String, Map<String, String>> injectionMap, org.apache.catalina.Context catalinaContext, ClassLoader containerClassLoader) {
         classLoader = catalinaContext.getLoader().getClassLoader();
@@ -112,13 +121,15 @@ public class DefaultInstanceManager implements InstanceManager {
             if (is != null) {
                 restrictedFilters.load(is);
             } else {
-                catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedFiltersResources"));
+                catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedFiltersResource"));
             }
         } catch (IOException e) {
             catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedServletsResources"), e);
         }
         this.context = context;
         this.injectionMap = injectionMap;
+        this.postConstructMethods = catalinaContext.findPostConstructMethods();
+        this.preDestroyMethods = catalinaContext.findPreDestroyMethods();
     }
 
     @Override
@@ -134,14 +145,14 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     @Override
-    public void newInstance(Object o) 
+    public void newInstance(Object o)
             throws IllegalAccessException, InvocationTargetException, NamingException {
         newInstance(o, o.getClass());
     }
 
     private Object newInstance(Object instance, Class<?> clazz) throws IllegalAccessException, InvocationTargetException, NamingException {
         if (!ignoreAnnotations) {
-            Map<String, String> injections = injectionMap.get(clazz.getName());
+            Map<String, String> injections = assembleInjectionsFromClassHierarchy(clazz);
             populateAnnotationsCache(clazz, injections);
             processAnnotations(instance, injections);
             postConstruct(instance, clazz);
@@ -149,6 +160,19 @@ public class DefaultInstanceManager implements InstanceManager {
         return instance;
     }
 
+    private Map<String, String> assembleInjectionsFromClassHierarchy(Class<?> clazz) {
+        Map<String, String> injections = new HashMap<String, String>();
+        Map<String, String> currentInjections = null;
+        while (clazz != null) {
+            currentInjections = this.injectionMap.get(clazz.getName());
+            if (currentInjections != null) {
+                injections.putAll(currentInjections);
+            }
+            clazz = clazz.getSuperclass();
+        }
+        return injections;
+    }
+
     @Override
     public void destroyInstance(Object instance) throws IllegalAccessException, InvocationTargetException {
         if (!ignoreAnnotations) {
@@ -179,15 +203,15 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // At the end the postconstruct annotated
         // method is invoked
-        List<AnnotationCacheEntry> annotations;
+        AnnotationCacheEntry[] annotations;
         synchronized (annotationCache) {
-            annotations = annotationCache.get(clazz).get();
+            annotations = annotationCache.get(clazz);
         }
         for (AnnotationCacheEntry entry : annotations) {
             if (entry.getType() == AnnotationCacheEntryType.POST_CONSTRUCT) {
-                Method postConstruct = (Method) entry.getAccessibleObject();
-                boolean accessibility = postConstruct.isAccessible();
+                Method postConstruct = getMethod(clazz, entry);
                 synchronized (postConstruct) {
+                    boolean accessibility = postConstruct.isAccessible();
                     postConstruct.setAccessible(true);
                     postConstruct.invoke(instance);
                     postConstruct.setAccessible(accessibility);
@@ -215,13 +239,9 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // At the end the postconstruct annotated
         // method is invoked
-        List<AnnotationCacheEntry> annotations = null;
+        AnnotationCacheEntry[] annotations = null;
         synchronized (annotationCache) {
-            WeakReference<List<AnnotationCacheEntry>> ref =
-                annotationCache.get(clazz);
-            if (ref != null) {
-                annotations = ref.get();
-            }
+            annotations = annotationCache.get(clazz);
         }
         if (annotations == null) {
             // instance not created through the instance manager
@@ -229,9 +249,9 @@ public class DefaultInstanceManager implements InstanceManager {
         }
         for (AnnotationCacheEntry entry : annotations) {
             if (entry.getType() == AnnotationCacheEntryType.PRE_DESTROY) {
-                Method preDestroy = (Method) entry.getAccessibleObject();
-                boolean accessibility = preDestroy.isAccessible();
+                Method preDestroy = getMethod(clazz, entry);
                 synchronized (preDestroy) {
+                    boolean accessibility = preDestroy.isAccessible();
                     preDestroy.setAccessible(true);
                     preDestroy.invoke(instance);
                     preDestroy.setAccessible(accessibility);
@@ -257,175 +277,159 @@ public class DefaultInstanceManager implements InstanceManager {
             Map<String, String> injections) throws IllegalAccessException,
             InvocationTargetException, NamingException {
 
-        if (context == null) {
-            // No resource injection
-            return;
-        }
+        List<AnnotationCacheEntry> annotations = null;
 
         while (clazz != null) {
-            List<AnnotationCacheEntry> annotations = null;
+            AnnotationCacheEntry[] annotationsArray = null;
             synchronized (annotationCache) {
-                WeakReference<List<AnnotationCacheEntry>> ref =
-                    annotationCache.get(clazz);
-                if (ref != null) {
-                    annotations = ref.get();
-                }
+                annotationsArray = annotationCache.get(clazz);
             }
-            if (annotations == null) {
-                annotations = new ArrayList<AnnotationCacheEntry>();
-                // Initialize fields annotations
-                Field[] fields = null;
-                if (Globals.IS_SECURITY_ENABLED) {
-                    final Class<?> clazz2 = clazz;
-                    fields = AccessController.doPrivileged(
-                            new PrivilegedAction<Field[]>(){
-                        @Override
-                        public Field[] run(){
-                            return clazz2.getDeclaredFields();
-                        }
-                    });
+            if (annotationsArray == null) {
+                if (annotations == null) {
+                    annotations = new ArrayList<AnnotationCacheEntry>();
                 } else {
-                    fields = clazz.getDeclaredFields();
+                    annotations.clear();
                 }
-                for (Field field : fields) {
-                    if (injections != null && injections.containsKey(field.getName())) {
-                        annotations.add(new AnnotationCacheEntry(field,
-                                injections.get(field.getName()),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (field.isAnnotationPresent(Resource.class)) {
-                        Resource annotation = field.getAnnotation(Resource.class);
-                        annotations.add(new AnnotationCacheEntry(field,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (field.isAnnotationPresent(EJB.class)) {
-                        EJB annotation = field.getAnnotation(EJB.class);
-                        annotations.add(new AnnotationCacheEntry(field,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (field.isAnnotationPresent(WebServiceRef.class)) {
-                        WebServiceRef annotation =
-                                field.getAnnotation(WebServiceRef.class);
-                        annotations.add(new AnnotationCacheEntry(field,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (field.isAnnotationPresent(PersistenceContext.class)) {
-                        PersistenceContext annotation =
-                                field.getAnnotation(PersistenceContext.class);
-                        annotations.add(new AnnotationCacheEntry(field,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (field.isAnnotationPresent(PersistenceUnit.class)) {
-                        PersistenceUnit annotation =
-                                field.getAnnotation(PersistenceUnit.class);
-                        annotations.add(new AnnotationCacheEntry(field,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    }
-                }
-        
-                // Initialize methods annotations
-                Method[] methods = null;
-                if (Globals.IS_SECURITY_ENABLED) {
-                    final Class<?> clazz2 = clazz;
-                    methods = AccessController.doPrivileged(
-                            new PrivilegedAction<Method[]>(){
-                        @Override
-                        public Method[] run(){
-                            return clazz2.getDeclaredMethods();
-                        }
-                    });
-                } else {
-                    methods = clazz.getDeclaredMethods();
-                }
-                Method postConstruct = null;
-                Method preDestroy = null;
-                for (Method method : methods) {
-                    String methodName = method.getName();
-                    if (injections != null && methodName.startsWith("set") && methodName.length() > 3) {
-                        String fieldName = Character.toLowerCase(methodName.charAt(3)) + methodName.substring(4);
-                        if (injections.containsKey(fieldName)) {
-                            annotations.add(new AnnotationCacheEntry(method,
-                                    injections.get(method.getName()),
+
+                if (context != null) {
+                    // Initialize fields annotations for resource injection if
+                    // JNDI is enabled
+                    Field[] fields = Introspection.getDeclaredFields(clazz);
+                    for (Field field : fields) {
+                        if (injections != null && injections.containsKey(field.getName())) {
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null,
+                                    injections.get(field.getName()),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (field.isAnnotationPresent(Resource.class)) {
+                            Resource annotation = field.getAnnotation(Resource.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null, annotation.name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (field.isAnnotationPresent(EJB.class)) {
+                            EJB annotation = field.getAnnotation(EJB.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null, annotation.name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (field.isAnnotationPresent(WebServiceRef.class)) {
+                            WebServiceRef annotation =
+                                    field.getAnnotation(WebServiceRef.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null, annotation.name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (field.isAnnotationPresent(PersistenceContext.class)) {
+                            PersistenceContext annotation =
+                                    field.getAnnotation(PersistenceContext.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null, annotation.name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (field.isAnnotationPresent(PersistenceUnit.class)) {
+                            PersistenceUnit annotation =
+                                    field.getAnnotation(PersistenceUnit.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    field.getName(), null, annotation.name(),
                                     AnnotationCacheEntryType.FIELD));
-                            break;
                         }
                     }
-                    if (method.isAnnotationPresent(Resource.class)) {
-                        Resource annotation = method.getAnnotation(Resource.class);
-                        annotations.add(new AnnotationCacheEntry(method,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (method.isAnnotationPresent(EJB.class)) {
-                        EJB annotation = method.getAnnotation(EJB.class);
-                        annotations.add(new AnnotationCacheEntry(method,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (method.isAnnotationPresent(WebServiceRef.class)) {
-                        WebServiceRef annotation =
-                                method.getAnnotation(WebServiceRef.class);
-                        annotations.add(new AnnotationCacheEntry(method,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (method.isAnnotationPresent(PersistenceContext.class)) {
-                        PersistenceContext annotation =
-                                method.getAnnotation(PersistenceContext.class);
-                        annotations.add(new AnnotationCacheEntry(method,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
-                    } else if (method.isAnnotationPresent(PersistenceUnit.class)) {
-                        PersistenceUnit annotation =
-                                method.getAnnotation(PersistenceUnit.class);
-                        annotations.add(new AnnotationCacheEntry(method,
-                                annotation.name(),
-                                AnnotationCacheEntryType.FIELD));
+                }
+
+                // Initialize methods annotations
+                Method[] methods = Introspection.getDeclaredMethods(clazz);
+                Method postConstruct = null;
+                String postConstructFromXml = postConstructMethods.get(clazz.getName());
+                Method preDestroy = null;
+                String preDestroyFromXml = preDestroyMethods.get(clazz.getName());
+                for (Method method : methods) {
+                    if (context != null) {
+                        // Resource injection only if JNDI is enabled
+                        if (injections != null &&
+                                Introspection.isValidSetter(method)) {
+                            String fieldName = Introspection.getPropertyName(method);
+                            if (injections.containsKey(fieldName)) {
+                                annotations.add(new AnnotationCacheEntry(
+                                        method.getName(),
+                                        method.getParameterTypes(),
+                                        injections.get(method.getName()),
+                                        AnnotationCacheEntryType.SETTER));
+                                break;
+                            }
+                        }
+                        if (method.isAnnotationPresent(Resource.class)) {
+                            Resource annotation = method.getAnnotation(Resource.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    method.getName(),
+                                    method.getParameterTypes(),
+                                    annotation.name(),
+                                    AnnotationCacheEntryType.SETTER));
+                        } else if (method.isAnnotationPresent(EJB.class)) {
+                            EJB annotation = method.getAnnotation(EJB.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    method.getName(),
+                                    method.getParameterTypes(),
+                                    annotation.name(),
+                                    AnnotationCacheEntryType.SETTER));
+                        } else if (method.isAnnotationPresent(WebServiceRef.class)) {
+                            WebServiceRef annotation =
+                                    method.getAnnotation(WebServiceRef.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    method.getName(),
+                                    method.getParameterTypes(),
+                                    annotation.name(),
+                                    AnnotationCacheEntryType.SETTER));
+                        } else if (method.isAnnotationPresent(PersistenceContext.class)) {
+                            PersistenceContext annotation =
+                                    method.getAnnotation(PersistenceContext.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    method.getName(),
+                                    method.getParameterTypes(),
+                                    annotation.name(),
+                                    AnnotationCacheEntryType.SETTER));
+                        } else if (method.isAnnotationPresent(PersistenceUnit.class)) {
+                            PersistenceUnit annotation =
+                                    method.getAnnotation(PersistenceUnit.class);
+                            annotations.add(new AnnotationCacheEntry(
+                                    method.getName(),
+                                    method.getParameterTypes(),
+                                    annotation.name(),
+                                    AnnotationCacheEntryType.SETTER));
+                        }
                     }
 
-                    if (method.isAnnotationPresent(PostConstruct.class)) {
-                        if ((postConstruct != null) ||
-                                (method.getParameterTypes().length != 0) ||
-                                (Modifier.isStatic(method.getModifiers())) ||
-                                (method.getExceptionTypes().length > 0) ||
-                                (!method.getReturnType().getName().equals("void"))) {
-                            throw new IllegalArgumentException(
-                                    "Invalid PostConstruct annotation");
-                        }
-                        postConstruct = method;
-                    }
-                    
-                    if (method.isAnnotationPresent(PreDestroy.class)) {
-                        if ((preDestroy != null ||
-                                method.getParameterTypes().length != 0) ||
-                                (Modifier.isStatic(method.getModifiers())) ||
-                                (method.getExceptionTypes().length > 0) ||
-                                (!method.getReturnType().getName().equals("void"))) {
-                            throw new IllegalArgumentException(
-                                    "Invalid PreDestroy annotation");
-                        }
-                        preDestroy = method;
-                    }
+                    postConstruct = findPostConstruct(postConstruct, postConstructFromXml, method);
+
+                    preDestroy = findPreDestroy(preDestroy, preDestroyFromXml, method);
                 }
+
                 if (postConstruct != null) {
-                    annotations.add(new AnnotationCacheEntry(postConstruct,
-                            null, AnnotationCacheEntryType.POST_CONSTRUCT));
+                    annotations.add(new AnnotationCacheEntry(
+                            postConstruct.getName(),
+                            postConstruct.getParameterTypes(), null,
+                            AnnotationCacheEntryType.POST_CONSTRUCT));
+                } else if (postConstructFromXml != null) {
+                    throw new IllegalArgumentException("Post construct method "
+                        + postConstructFromXml + " for class " + clazz.getName()
+                        + " is declared in deployment descriptor but cannot be found.");
                 }
                 if (preDestroy != null) {
-                    annotations.add(new AnnotationCacheEntry(preDestroy,
-                            null, AnnotationCacheEntryType.PRE_DESTROY));
+                    annotations.add(new AnnotationCacheEntry(
+                            preDestroy.getName(),
+                            preDestroy.getParameterTypes(), null,
+                            AnnotationCacheEntryType.PRE_DESTROY));
+                } else if (preDestroyFromXml != null) {
+                    throw new IllegalArgumentException("Pre destroy method "
+                        + preDestroyFromXml + " for class " + clazz.getName()
+                        + " is declared in deployment descriptor but cannot be found.");
                 }
-                if (annotations.size() == 0) {
-                    // Use common empty list to save memory 
-                    annotations = Collections.emptyList();
+                if (annotations.isEmpty()) {
+                    // Use common object to save memory
+                    annotationsArray = ANNOTATIONS_EMPTY;
+                } else {
+                    annotationsArray = annotations.toArray(
+                            new AnnotationCacheEntry[annotations.size()]);
                 }
                 synchronized (annotationCache) {
-                    annotationCache.put(clazz,
-                            new WeakReference<List<AnnotationCacheEntry>>(
-                                    annotations));
+                    annotationCache.put(clazz, annotationsArray);
                 }
-            } else {
-                // If the annotations for this class have been cached, the
-                // annotations for all the super classes will have been cachced
-                // as well
-                break;
             }
             clazz = clazz.getSuperclass();
         }
@@ -451,23 +455,21 @@ public class DefaultInstanceManager implements InstanceManager {
         }
 
         Class<?> clazz = instance.getClass();
-        
+
         while (clazz != null) {
-            List<AnnotationCacheEntry> annotations;
+            AnnotationCacheEntry[] annotations;
             synchronized (annotationCache) {
-                annotations = annotationCache.get(clazz).get();
+                annotations = annotationCache.get(clazz);
             }
             for (AnnotationCacheEntry entry : annotations) {
-                if (entry.getType() == AnnotationCacheEntryType.FIELD) {
-                    if (entry.getAccessibleObject() instanceof Method) {
-                        lookupMethodResource(context, instance,
-                                (Method) entry.getAccessibleObject(),
-                                entry.getName(), clazz);
-                    } else {
-                        lookupFieldResource(context, instance,
-                                (Field) entry.getAccessibleObject(),
-                                entry.getName(), clazz);
-                    }
+                if (entry.getType() == AnnotationCacheEntryType.SETTER) {
+                    lookupMethodResource(context, instance,
+                            getMethod(clazz, entry),
+                            entry.getName(), clazz);
+                } else if (entry.getType() == AnnotationCacheEntryType.FIELD) {
+                    lookupFieldResource(context, instance,
+                            getField(clazz, entry),
+                            entry.getName(), clazz);
                 }
             }
             clazz = clazz.getSuperclass();
@@ -526,7 +528,9 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     private void checkAccess(Class<?> clazz) {
-        if (privileged) return;
+        if (privileged) {
+            return;
+        }
         if (Filter.class.isAssignableFrom(clazz)) {
             checkAccess(clazz, restrictedFilters);
         } else if (Servlet.class.isAssignableFrom(clazz)) {
@@ -577,8 +581,8 @@ public class DefaultInstanceManager implements InstanceManager {
                 context.lookup(clazz.getName() + "/" + field.getName());
         }
 
-        accessibility = field.isAccessible();
         synchronized (field) {
+            accessibility = field.isAccessible();
             field.setAccessible(true);
             field.set(instance, lookedupResource);
             field.setAccessible(accessibility);
@@ -602,11 +606,9 @@ public class DefaultInstanceManager implements InstanceManager {
             Object instance, Method method, String name, Class<?> clazz)
             throws NamingException, IllegalAccessException, InvocationTargetException {
 
-        if (!method.getName().startsWith("set")
-                || method.getName().length() < 4
-                || method.getParameterTypes().length != 1
-                || !method.getReturnType().getName().equals("void")) {
-            throw new IllegalArgumentException("Invalid method resource injection annotation");
+        if (!Introspection.isValidSetter(method)) {
+            throw new IllegalArgumentException(
+                    sm.getString("defaultInstanceManager.invalidInjection"));
         }
 
         Object lookedupResource;
@@ -618,29 +620,24 @@ public class DefaultInstanceManager implements InstanceManager {
             lookedupResource = context.lookup(normalizedName);
         } else {
             lookedupResource = context.lookup(
-                    clazz.getName() + "/" + getName(method));
+                    clazz.getName() + "/" + Introspection.getPropertyName(method));
         }
 
-        accessibility = method.isAccessible();
         synchronized (method) {
+            accessibility = method.isAccessible();
             method.setAccessible(true);
             method.invoke(instance, lookedupResource);
             method.setAccessible(accessibility);
         }
     }
 
+    @Deprecated
     public static String getName(Method setter) {
-        StringBuilder name = new StringBuilder(setter.getName());
-
-        // remove 'set'
-        name.delete(0, 3);
-
-        // lowercase first char
-        name.setCharAt(0, Character.toLowerCase(name.charAt(0)));
-
-        return name.toString();
+        // Note: method signature has already been checked for correctness.
+        // The method name always starts with "set".
+        return Introspector.decapitalize(setter.getName().substring(3));
     }
-    
+
     private static String normalize(String jndiName){
         if(jndiName != null && jndiName.startsWith("java:comp/env/")){
             return jndiName.substring(14);
@@ -648,20 +645,126 @@ public class DefaultInstanceManager implements InstanceManager {
         return jndiName;
     }
 
+    private static Method getMethod(final Class<?> clazz,
+            final AnnotationCacheEntry entry) {
+        Method result = null;
+        if (Globals.IS_SECURITY_ENABLED) {
+            result = AccessController.doPrivileged(
+                    new PrivilegedAction<Method>() {
+                        @Override
+                        public Method run() {
+                            Method result = null;
+                            try {
+                                result = clazz.getDeclaredMethod(
+                                        entry.getAccessibleObjectName(),
+                                        entry.getParamTypes());
+                            } catch (NoSuchMethodException e) {
+                                // Should never happen. On that basis don't log
+                                // it.
+                            }
+                            return result;
+                        }
+            });
+        } else {
+            try {
+                result = clazz.getDeclaredMethod(
+                        entry.getAccessibleObjectName(), entry.getParamTypes());
+            } catch (NoSuchMethodException e) {
+                // Should never happen. On that basis don't log it.
+            }
+        }
+        return result;
+    }
+
+    private static Field getField(final Class<?> clazz,
+            final AnnotationCacheEntry entry) {
+        Field result = null;
+        if (Globals.IS_SECURITY_ENABLED) {
+            result = AccessController.doPrivileged(
+                    new PrivilegedAction<Field>() {
+                        @Override
+                        public Field run() {
+                            Field result = null;
+                            try {
+                                result = clazz.getDeclaredField(
+                                        entry.getAccessibleObjectName());
+                            } catch (NoSuchFieldException e) {
+                                // Should never happen. On that basis don't log
+                                // it.
+                            }
+                            return result;
+                        }
+            });
+        } else {
+            try {
+                result = clazz.getDeclaredField(
+                        entry.getAccessibleObjectName());
+            } catch (NoSuchFieldException e) {
+                // Should never happen. On that basis don't log it.
+            }
+        }
+        return result;
+    }
+
+
+    private static Method findPostConstruct(Method currentPostConstruct,
+            String postConstructFromXml, Method method) {
+        return findLifecycleCallback(currentPostConstruct,
+            postConstructFromXml, method, PostConstruct.class);
+    }
+
+    private static Method findPreDestroy(Method currentPreDestroy,
+        String preDestroyFromXml, Method method) {
+        return findLifecycleCallback(currentPreDestroy,
+            preDestroyFromXml, method, PreDestroy.class);
+    }
+
+    private static Method findLifecycleCallback(Method currentMethod,
+            String methodNameFromXml, Method method,
+            Class<? extends Annotation> annotation) {
+        Method result = currentMethod;
+        if (methodNameFromXml != null) {
+            if (method.getName().equals(methodNameFromXml)) {
+                if (!Introspection.isValidLifecycleCallback(method)) {
+                    throw new IllegalArgumentException(
+                        "Invalid " + annotation.getName() + " annotation");
+                }
+                result = method;
+            }
+        } else {
+            if (method.isAnnotationPresent(annotation)) {
+                if (currentMethod != null ||
+                    !Introspection.isValidLifecycleCallback(method)) {
+                    throw new IllegalArgumentException(
+                        "Invalid " + annotation.getName() + " annotation");
+                }
+                result = method;
+            }
+        }
+        return result;
+    }
+
     private static final class AnnotationCacheEntry {
-        private final AccessibleObject accessibleObject;
+        private final String accessibleObjectName;
+        private final Class<?>[] paramTypes;
         private final String name;
         private final AnnotationCacheEntryType type;
 
-        public AnnotationCacheEntry(AccessibleObject accessibleObject,
-                String name, AnnotationCacheEntryType type) {
-            this.accessibleObject = accessibleObject;
+        public AnnotationCacheEntry(String accessibleObjectName,
+                Class<?>[] paramTypes, String name,
+                AnnotationCacheEntryType type) {
+            this.accessibleObjectName = accessibleObjectName;
+            this.paramTypes = paramTypes;
             this.name = name;
             this.type = type;
         }
 
-        public AccessibleObject getAccessibleObject() {
-            return accessibleObject;
+        public String getAccessibleObjectName() {
+            return accessibleObjectName;
+        }
+
+        public Class<?>[] getParamTypes() {
+            return paramTypes;
         }
 
         public String getName() {
@@ -673,6 +776,6 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     private static enum AnnotationCacheEntryType {
-        FIELD, POST_CONSTRUCT, PRE_DESTROY
+        FIELD, SETTER, POST_CONSTRUCT, PRE_DESTROY
     }
 }
