@@ -518,20 +518,6 @@ public class InternalAprInputBuffer extends AbstractInputBuffer<Long> {
     }
     
     
-    /**
-     * Available bytes (note that due to encoding, this may not correspond )
-     */
-    public int available() {
-        int result = (lastValid - pos);
-        if ((result == 0) && (lastActiveFilter >= 0)) {
-            for (int i = 0; (result == 0) && (i <= lastActiveFilter); i++) {
-                result = activeFilters[i].available();
-            }
-        }
-        return result;
-    }
-
-
     // ---------------------------------------------------- InputBuffer Methods
 
 
