@@ -17,10 +17,8 @@
 
 package org.apache.jasper.compiler;
 
-import java.io.InputStream;
 import java.util.Iterator;
 import java.util.Vector;
-import java.net.URL;
 
 import javax.servlet.ServletContext;
 
@@ -29,7 +27,6 @@ import org.apache.jasper.xmlparser.ParserUtils;
 import org.apache.jasper.xmlparser.TreeNode;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.xml.sax.InputSource;
 
 /**
  * Handles the jsp-config element in WEB_INF/web.xml.  This is used
@@ -41,20 +38,21 @@ import org.xml.sax.InputSource;
 
 public class JspConfig {
 
-    private static final String WEB_XML = "/WEB-INF/web.xml";
-
     // Logger
-    private Log log = LogFactory.getLog(JspConfig.class);
+    private final Log log = LogFactory.getLog(JspConfig.class);
 
-    private Vector jspProperties = null;
+    private Vector<JspPropertyGroup> jspProperties = null;
     private ServletContext ctxt;
     private volatile boolean initialized = false;
 
-    private final static String defaultIsXml = null;		// unspecified
-    private String defaultIsELIgnored = null;	// unspecified
-    private final static String defaultIsScriptingInvalid = null;
+    private static final String defaultIsXml = null;    // unspecified
+    private String defaultIsELIgnored = null;           // unspecified
+    private static final String defaultIsScriptingInvalid = null;
     private String defaultDeferedSyntaxAllowedAsLiteral = null;
-    private final static String defaultTrimDirectiveWhitespaces = null;
+    private static final String defaultTrimDirectiveWhitespaces = null;
+    private static final String defaultDefaultContentType = null;
+    private static final String defaultBuffer = null;
+    private static final String defaultErrorOnUndeclaredNamespace = "false";
     private JspProperty defaultJspProperty;
 
     public JspConfig(ServletContext ctxt) {
@@ -72,24 +70,20 @@ public class JspConfig {
         return 2.3;
     }
 
-    private void processWebDotXml(ServletContext ctxt) throws JasperException {
+    private void processWebDotXml() throws JasperException {
 
-        InputStream is = null;
+        WebXml webXml = null;
 
         try {
-            URL uri = ctxt.getResource(WEB_XML);
-            if (uri == null) {
-                // no web.xml
-                return;
+            webXml = new WebXml(ctxt);
+            
+            TreeNode webApp = null;
+            if (webXml.getInputSource() != null) {
+                ParserUtils pu = new ParserUtils();
+                webApp = pu.parseXMLDocument(webXml.getSystemId(),
+                        webXml.getInputSource());
             }
 
-            is = uri.openStream();
-            InputSource ip = new InputSource(is);
-            ip.setSystemId(uri.toExternalForm()); 
-
-            ParserUtils pu = new ParserUtils();
-            TreeNode webApp = pu.parseXMLDocument(WEB_XML, ip);
-
             if (webApp == null
                     || getVersion(webApp) < 2.4) {
                 defaultIsELIgnored = "true";
@@ -104,26 +98,30 @@ public class JspConfig {
                 return;
             }
 
-            jspProperties = new Vector();
-            Iterator jspPropertyList = jspConfig.findChildren("jsp-property-group");
+            jspProperties = new Vector<JspPropertyGroup>();
+            Iterator<TreeNode> jspPropertyList =
+                jspConfig.findChildren("jsp-property-group");
             while (jspPropertyList.hasNext()) {
 
-                TreeNode element = (TreeNode) jspPropertyList.next();
-                Iterator list = element.findChildren();
+                TreeNode element = jspPropertyList.next();
+                Iterator<TreeNode> list = element.findChildren();
 
-                Vector urlPatterns = new Vector();
+                Vector<String> urlPatterns = new Vector<String>();
                 String pageEncoding = null;
                 String scriptingInvalid = null;
                 String elIgnored = null;
                 String isXml = null;
-                Vector includePrelude = new Vector();
-                Vector includeCoda = new Vector();
+                Vector<String> includePrelude = new Vector<String>();
+                Vector<String> includeCoda = new Vector<String>();
                 String deferredSyntaxAllowedAsLiteral = null;
                 String trimDirectiveWhitespaces = null;
+                String defaultContentType = null;
+                String buffer = null;
+                String errorOnUndeclaredNamespace = null;
 
                 while (list.hasNext()) {
 
-                    element = (TreeNode) list.next();
+                    element = list.next();
                     String tname = element.getName();
 
                     if ("url-pattern".equals(tname))
@@ -144,6 +142,12 @@ public class JspConfig {
                         deferredSyntaxAllowedAsLiteral = element.getBody();
                     else if ("trim-directive-whitespaces".equals(tname))
                         trimDirectiveWhitespaces = element.getBody();
+                    else if ("default-content-type".equals(tname))
+                        defaultContentType = element.getBody();
+                    else if ("buffer".equals(tname))
+                        buffer = element.getBody();
+                    else if ("error-on-undeclared-namespace".equals(tname))
+                        errorOnUndeclaredNamespace = element.getBody();
                 }
 
                 if (urlPatterns.size() == 0) {
@@ -153,7 +157,7 @@ public class JspConfig {
                 // Add one JspPropertyGroup for each URL Pattern.  This makes
                 // the matching logic easier.
                 for( int p = 0; p < urlPatterns.size(); p++ ) {
-                    String urlPattern = (String)urlPatterns.elementAt( p );
+                    String urlPattern = urlPatterns.elementAt( p );
                     String path = null;
                     String extension = null;
 
@@ -177,7 +181,7 @@ public class JspConfig {
                             extension = file.substring(file.indexOf('.')+1);
                         }
 
-                        // The url patterns are reconstructed as the follwoing:
+                        // The url patterns are reconstructed as the following:
                         // path != null, extension == null:  / or /foo/bar.ext
                         // path == null, extension != null:  *.ext
                         // path != null, extension == "*":   /foo/*
@@ -200,7 +204,10 @@ public class JspConfig {
                             includePrelude,
                             includeCoda,
                             deferredSyntaxAllowedAsLiteral,
-                            trimDirectiveWhitespaces);
+                            trimDirectiveWhitespaces,
+                            defaultContentType,
+                            buffer,
+                            errorOnUndeclaredNamespace);
                     JspPropertyGroup propertyGroup =
                         new JspPropertyGroup(path, extension, property);
 
@@ -210,10 +217,8 @@ public class JspConfig {
         } catch (Exception ex) {
             throw new JasperException(ex);
         } finally {
-            if (is != null) {
-                try {
-                    is.close();
-                } catch (Throwable t) {}
+            if (webXml != null) {
+                webXml.close();
             }
         }
     }
@@ -223,13 +228,16 @@ public class JspConfig {
         if (!initialized) {
             synchronized (this) {
                 if (!initialized) {
-                    processWebDotXml(ctxt);
+                    processWebDotXml();
                     defaultJspProperty = new JspProperty(defaultIsXml,
                             defaultIsELIgnored,
                             defaultIsScriptingInvalid,
                             null, null, null,
                             defaultDeferedSyntaxAllowedAsLiteral, 
-                            defaultTrimDirectiveWhitespaces);
+                            defaultTrimDirectiveWhitespaces,
+                            defaultDefaultContentType,
+                            defaultBuffer,
+                            defaultErrorOnUndeclaredNamespace);
                     initialized = true;
                 }
             }
@@ -281,7 +289,7 @@ public class JspConfig {
 
         init();
 
-        // JSP Configuration settings do not apply to tag files	    
+        // JSP Configuration settings do not apply to tag files
         if (jspProperties == null || uri.endsWith(".tag")
                 || uri.endsWith(".tagx")) {
             return defaultJspProperty;
@@ -298,8 +306,8 @@ public class JspConfig {
             uriExtension = uri.substring(index+1);
         }
 
-        Vector includePreludes = new Vector();
-        Vector includeCodas = new Vector();
+        Vector<String> includePreludes = new Vector<String>();
+        Vector<String> includeCodas = new Vector<String>();
 
         JspPropertyGroup isXmlMatch = null;
         JspPropertyGroup elIgnoredMatch = null;
@@ -307,11 +315,14 @@ public class JspConfig {
         JspPropertyGroup pageEncodingMatch = null;
         JspPropertyGroup deferedSyntaxAllowedAsLiteralMatch = null;
         JspPropertyGroup trimDirectiveWhitespacesMatch = null;
+        JspPropertyGroup defaultContentTypeMatch = null;
+        JspPropertyGroup bufferMatch = null;
+        JspPropertyGroup errorOnUndeclaredNamespaceMatch = null;
 
-        Iterator iter = jspProperties.iterator();
+        Iterator<JspPropertyGroup> iter = jspProperties.iterator();
         while (iter.hasNext()) {
 
-            JspPropertyGroup jpg = (JspPropertyGroup) iter.next();
+            JspPropertyGroup jpg = iter.next();
             JspProperty jp = jpg.getJspProperty();
 
             // (arrays will be the same length)
@@ -369,6 +380,17 @@ public class JspConfig {
                 trimDirectiveWhitespacesMatch =
                     selectProperty(trimDirectiveWhitespacesMatch, jpg);
             }
+            if (jp.getDefaultContentType() != null) {
+                defaultContentTypeMatch =
+                    selectProperty(defaultContentTypeMatch, jpg);
+            }
+            if (jp.getBuffer() != null) {
+                bufferMatch = selectProperty(bufferMatch, jpg);
+            }
+            if (jp.isErrorOnUndeclaredNamespace() != null) {
+                errorOnUndeclaredNamespaceMatch =
+                    selectProperty(errorOnUndeclaredNamespaceMatch, jpg);
+            }
         }
 
 
@@ -376,8 +398,12 @@ public class JspConfig {
         String isELIgnored = defaultIsELIgnored;
         String isScriptingInvalid = defaultIsScriptingInvalid;
         String pageEncoding = null;
-        String isDeferedSyntaxAllowedAsLiteral = defaultDeferedSyntaxAllowedAsLiteral;
+        String isDeferedSyntaxAllowedAsLiteral =
+            defaultDeferedSyntaxAllowedAsLiteral;
         String isTrimDirectiveWhitespaces = defaultTrimDirectiveWhitespaces;
+        String defaultContentType = defaultDefaultContentType;
+        String buffer = defaultBuffer;
+        String errorOnUndelcaredNamespace = defaultErrorOnUndeclaredNamespace;
 
         if (isXmlMatch != null) {
             isXml = isXmlMatch.getJspProperty().isXml();
@@ -400,10 +426,22 @@ public class JspConfig {
             isTrimDirectiveWhitespaces =
                 trimDirectiveWhitespacesMatch.getJspProperty().isTrimDirectiveWhitespaces();
         }
+        if (defaultContentTypeMatch != null) {
+            defaultContentType =
+                defaultContentTypeMatch.getJspProperty().getDefaultContentType();
+        }
+        if (bufferMatch != null) {
+            buffer = bufferMatch.getJspProperty().getBuffer();
+        }
+        if (errorOnUndeclaredNamespaceMatch != null) {
+            errorOnUndelcaredNamespace =
+                errorOnUndeclaredNamespaceMatch.getJspProperty().isErrorOnUndeclaredNamespace();
+        }
 
         return new JspProperty(isXml, isELIgnored, isScriptingInvalid,
                 pageEncoding, includePreludes, includeCodas, 
-                isDeferedSyntaxAllowedAsLiteral, isTrimDirectiveWhitespaces);
+                isDeferedSyntaxAllowedAsLiteral, isTrimDirectiveWhitespaces,
+                defaultContentType, buffer, errorOnUndelcaredNamespace);
     }
 
     /**
@@ -428,11 +466,10 @@ public class JspConfig {
             uriExtension = uri.substring(index+1);
         }
 
-        Iterator iter = jspProperties.iterator();
+        Iterator<JspPropertyGroup> iter = jspProperties.iterator();
         while (iter.hasNext()) {
 
-            JspPropertyGroup jpg = (JspPropertyGroup) iter.next();
-            JspProperty jp = jpg.getJspProperty();
+            JspPropertyGroup jpg = iter.next();
 
             String extension = jpg.getExtension();
             String path = jpg.getPath();
@@ -453,7 +490,7 @@ public class JspConfig {
         return false;
     }
 
-    static class JspPropertyGroup {
+    public static class JspPropertyGroup {
         private String path;
         private String extension;
         private JspProperty jspProperty;
@@ -478,22 +515,28 @@ public class JspConfig {
         }
     }
 
-    static public class JspProperty {
+    public static class JspProperty {
 
         private String isXml;
         private String elIgnored;
         private String scriptingInvalid;
         private String pageEncoding;
-        private Vector includePrelude;
-        private Vector includeCoda;
+        private Vector<String> includePrelude;
+        private Vector<String> includeCoda;
         private String deferedSyntaxAllowedAsLiteral;
         private String trimDirectiveWhitespaces;
+        private String defaultContentType;
+        private String buffer;
+        private String errorOnUndeclaredNamespace;
 
         public JspProperty(String isXml, String elIgnored,
                 String scriptingInvalid, String pageEncoding,
-                Vector includePrelude, Vector includeCoda,
+                Vector<String> includePrelude, Vector<String> includeCoda,
                 String deferedSyntaxAllowedAsLiteral, 
-                String trimDirectiveWhitespaces) {
+                String trimDirectiveWhitespaces,
+                String defaultContentType,
+                String buffer,
+                String errorOnUndeclaredNamespace) {
 
             this.isXml = isXml;
             this.elIgnored = elIgnored;
@@ -503,6 +546,9 @@ public class JspConfig {
             this.includeCoda = includeCoda;
             this.deferedSyntaxAllowedAsLiteral = deferedSyntaxAllowedAsLiteral;
             this.trimDirectiveWhitespaces = trimDirectiveWhitespaces;
+            this.defaultContentType = defaultContentType;
+            this.buffer = buffer;
+            this.errorOnUndeclaredNamespace = errorOnUndeclaredNamespace;
         }
 
         public String isXml() {
@@ -521,11 +567,11 @@ public class JspConfig {
             return pageEncoding;
         }
 
-        public Vector getIncludePrelude() {
+        public Vector<String> getIncludePrelude() {
             return includePrelude;
         }
 
-        public Vector getIncludeCoda() {
+        public Vector<String> getIncludeCoda() {
             return includeCoda;
         }
         
@@ -536,5 +582,17 @@ public class JspConfig {
         public String isTrimDirectiveWhitespaces() {
             return trimDirectiveWhitespaces;
         }
+        
+        public String getDefaultContentType() {
+            return defaultContentType;
+        }
+        
+        public String getBuffer() {
+            return buffer;
+        }
+        
+        public String isErrorOnUndeclaredNamespace() {
+            return errorOnUndeclaredNamespace;
+        }
     }
 }
