@@ -49,11 +49,17 @@ import static org.junit.Assert.fail;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Lifecycle;
+import org.apache.catalina.LifecycleEvent;
+import org.apache.catalina.LifecycleException;
+import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.authenticator.BasicAuthenticator;
 import org.apache.catalina.deploy.FilterDef;
 import org.apache.catalina.deploy.FilterMap;
 import org.apache.catalina.deploy.LoginConfig;
+import org.apache.catalina.loader.WebappLoader;
 import org.apache.catalina.startup.SimpleHttpClient;
 import org.apache.catalina.startup.TestTomcat.MapRealm;
 import org.apache.catalina.startup.Tomcat;
@@ -70,6 +76,10 @@ public class TestStandardContext extends TomcatBaseTest {
 
     @Test
     public void testBug46243() throws Exception {
+        // This tests that if a Filter init() fails then the web application
+        // is not put into service. (BZ 46243)
+        // This also tests that if the cause of the failure is gone,
+        // the context can be started without a need to redeploy it.
 
         // Set up a container
         Tomcat tomcat = getTomcatInstance();
@@ -80,23 +90,7 @@ public class TestStandardContext extends TomcatBaseTest {
         }
 
         Context root = tomcat.addContext("", "ROOT");
-
-        // Add test a filter that fails
-        FilterDef filterDef = new FilterDef();
-        filterDef.setFilterClass(Bug46243Filter.class.getName());
-        filterDef.setFilterName("Bug46243");
-        root.addFilterDef(filterDef);
-        FilterMap filterMap = new FilterMap();
-        filterMap.setFilterName("Bug46243");
-        filterMap.addURLPattern("*");
-        root.addFilterMap(filterMap);
-
-        // Add a test servlet so there is something to generate a response if
-        // it works (although it shouldn't)
-        Tomcat.addServlet(root, "Bug46243", new HelloWorldServlet());
-        root.addServletMapping("/", "Bug46243");
-
-
+        configureTest46243Context(root, true);
         tomcat.start();
 
         // Configure the client
@@ -107,6 +101,41 @@ public class TestStandardContext extends TomcatBaseTest {
         client.connect();
         client.processRequest();
         assertTrue(client.isResponse404());
+
+        // Context failed to start. This checks that automatic transition
+        // from FAILED to STOPPED state was successful.
+        assertEquals(LifecycleState.STOPPED, root.getState());
+
+        // Prepare context for the second attempt
+        // Configuration was cleared on stop() thanks to
+        // StandardContext.resetContext(), so we need to configure it again
+        // from scratch.
+        configureTest46243Context(root, false);
+        root.start();
+        // The same request is processed successfully
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        assertEquals(Bug46243Filter.class.getName()
+                + HelloWorldServlet.RESPONSE_TEXT, client.getResponseBody());
+    }
+
+    private static void configureTest46243Context(Context context, boolean fail) {
+        // Add a test filter that fails
+        FilterDef filterDef = new FilterDef();
+        filterDef.setFilterClass(Bug46243Filter.class.getName());
+        filterDef.setFilterName("Bug46243");
+        filterDef.addInitParameter("fail", Boolean.toString(fail));
+        context.addFilterDef(filterDef);
+        FilterMap filterMap = new FilterMap();
+        filterMap.setFilterName("Bug46243");
+        filterMap.addURLPattern("*");
+        context.addFilterMap(filterMap);
+
+        // Add a test servlet so there is something to generate a response if
+        // it works (although it shouldn't)
+        Tomcat.addServlet(context, "Bug46243", new HelloWorldServlet());
+        context.addServletMapping("/", "Bug46243");
     }
 
     private static final class Bug46243Client extends SimpleHttpClient {
@@ -132,19 +161,127 @@ public class TestStandardContext extends TomcatBaseTest {
         @Override
         public void doFilter(ServletRequest request, ServletResponse response,
                 FilterChain chain) throws IOException, ServletException {
-            // If it works, do nothing
+            PrintWriter out = response.getWriter();
+            out.print(getClass().getName());
             chain.doFilter(request, response);
         }
 
         @Override
         public void init(FilterConfig filterConfig) throws ServletException {
-            throw new ServletException("Init fail", new ClassNotFoundException());
+            boolean fail = filterConfig.getInitParameter("fail").equals("true");
+            if (fail) {
+                throw new ServletException("Init fail (test)",
+                        new ClassNotFoundException());
+            }
         }
 
     }
 
+    @Test
+    public void testWebappLoaderStartFail() throws Exception {
+        // Test that if WebappLoader start() fails and if the cause of
+        // the failure is gone, the context can be started without
+        // a need to redeploy it.
+
+        // Set up a container
+        Tomcat tomcat = getTomcatInstance();
+        tomcat.start();
+        // To not start Context automatically, as we have to configure it first
+        ((ContainerBase) tomcat.getHost()).setStartChildren(false);
+
+        FailingWebappLoader loader = new FailingWebappLoader();
+        File root = new File("test/webapp-3.0");
+        Context context = tomcat.addWebapp("", root.getAbsolutePath());
+        context.setLoader(loader);
+
+        try {
+            context.start();
+            fail();
+        } catch (LifecycleException ex) {
+            // As expected
+        }
+        assertEquals(LifecycleState.FAILED, context.getState());
+
+        // The second attempt
+        loader.setFail(false);
+        context.start();
+        assertEquals(LifecycleState.STARTED, context.getState());
+
+        // Using a test from testBug49922() to check that the webapp is running
+        ByteChunk result = getUrl("http://localhost:" + getPort() +
+                "/bug49922/target");
+        assertEquals("Target", result.toString());
+    }
+
+    @Test
+    public void testWebappListenerConfigureFail() throws Exception {
+        // Test that if LifecycleListener on webapp fails during
+        // configure_start event and if the cause of the failure is gone,
+        // the context can be started without a need to redeploy it.
+
+        // Set up a container
+        Tomcat tomcat = getTomcatInstance();
+        tomcat.start();
+        // To not start Context automatically, as we have to configure it first
+        ((ContainerBase) tomcat.getHost()).setStartChildren(false);
+
+        FailingLifecycleListener listener = new FailingLifecycleListener();
+        File root = new File("test/webapp-3.0");
+        Context context = tomcat.addWebapp("", root.getAbsolutePath());
+        context.addLifecycleListener(listener);
+
+        try {
+            context.start();
+            fail();
+        } catch (LifecycleException ex) {
+            // As expected
+        }
+        assertEquals(LifecycleState.FAILED, context.getState());
+
+        // The second attempt
+        listener.setFail(false);
+        context.start();
+        assertEquals(LifecycleState.STARTED, context.getState());
+
+        // Using a test from testBug49922() to check that the webapp is running
+        ByteChunk result = getUrl("http://localhost:" + getPort() +
+                "/bug49922/target");
+        assertEquals("Target", result.toString());
+    }
+
+    private static class FailingWebappLoader extends WebappLoader {
+        private boolean fail = true;
+        protected void setFail(boolean fail) {
+            this.fail = fail;
+        }
+        @Override
+        protected void startInternal() throws LifecycleException {
+            if (fail) {
+                throw new RuntimeException("Start fail (test)");
+            }
+            super.startInternal();
+        }
+    }
+
+    private static class FailingLifecycleListener implements LifecycleListener {
+        private final String failEvent = Lifecycle.CONFIGURE_START_EVENT;
+        private boolean fail = true;
+        protected void setFail(boolean fail) {
+            this.fail = fail;
+        }
+        @Override
+        public void lifecycleEvent(LifecycleEvent event) {
+            if (fail && event.getType().equals(failEvent)) {
+                throw new RuntimeException(failEvent + " fail (test)");
+            }
+        }
+    }
+
     @Test
     public void testBug49922() throws Exception {
+        // Test that filter mapping works. Test that the same filter is
+        // called only once, even if is selected by several mapping
+        // url-patterns or by both a url-pattern and a servlet-name.
 
         // Set up a container
         Tomcat tomcat = getTomcatInstance();
@@ -269,6 +406,9 @@ public class TestStandardContext extends TomcatBaseTest {
 
     @Test
     public void testBug50015() throws Exception {
+        // Test that configuring servlet security constraints programmatically
+        // does work.
+
         // Set up a container
         Tomcat tomcat = getTomcatInstance();
 
@@ -348,6 +488,9 @@ public class TestStandardContext extends TomcatBaseTest {
     }
 
     private void doTestBug51376(boolean loadOnStartUp) throws Exception {
+        // Test that for a servlet that was added programmatically its
+        // loadOnStartup property is honored and its init() and destroy()
+        // methods are called.
 
         // Set up a container
         Tomcat tomcat = getTomcatInstance();
@@ -368,6 +511,7 @@ public class TestStandardContext extends TomcatBaseTest {
 
         // Make sure that init() and destroy() were called correctly
         assertTrue(sci.getServlet().isOk());
+        assertTrue(loadOnStartUp == sci.getServlet().isInitCalled());
     }
 
     public static final class Bug51376SCI
@@ -402,11 +546,11 @@ public class TestStandardContext extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
 
         private Boolean initOk = null;
-        private Boolean destoryOk = null;
+        private Boolean destroyOk = null;
 
         @Override
         public void init() {
-            if (initOk == null && destoryOk == null) {
+            if (initOk == null && destroyOk == null) {
                 initOk = Boolean.TRUE;
             } else {
                 initOk = Boolean.FALSE;
@@ -415,10 +559,10 @@ public class TestStandardContext extends TomcatBaseTest {
 
         @Override
         public void destroy() {
-            if (initOk.booleanValue() && destoryOk == null) {
-                destoryOk = Boolean.TRUE;
+            if (initOk.booleanValue() && destroyOk == null) {
+                destroyOk = Boolean.TRUE;
             } else {
-                destoryOk = Boolean.FALSE;
+                destroyOk = Boolean.FALSE;
             }
         }
 
@@ -430,15 +574,19 @@ public class TestStandardContext extends TomcatBaseTest {
         }
 
         protected boolean isOk() {
-            if (initOk != null && initOk.booleanValue() && destoryOk != null &&
-                    destoryOk.booleanValue()) {
+            if (initOk != null && initOk.booleanValue() && destroyOk != null &&
+                    destroyOk.booleanValue()) {
                 return true;
-            } else if (initOk == null && destoryOk == null) {
+            } else if (initOk == null && destroyOk == null) {
                 return true;
             } else {
                 return false;
             }
         }
+
+        protected boolean isInitCalled() {
+            return initOk != null && initOk.booleanValue();
+        }
     }
 
     /**
@@ -604,4 +752,38 @@ public class TestStandardContext extends TomcatBaseTest {
             return false; // Don't care
         }
     }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPostConstructMethodNullClassName() {
+        new StandardContext().addPostConstructMethod(null, "");
+    }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPostConstructMethodNullMethodName() {
+        new StandardContext().addPostConstructMethod("", null);
+    }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPostConstructMethodConflicts() {
+        StandardContext standardContext = new StandardContext();
+        standardContext.addPostConstructMethod("a", "a");
+        standardContext.addPostConstructMethod("a", "b");
+    }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPreDestroyMethodNullClassName() {
+        new StandardContext().addPreDestroyMethod(null, "");
+    }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPreDestroyMethodNullMethodName() {
+        new StandardContext().addPreDestroyMethod("", null);
+    }
+
+    @Test(expected = IllegalArgumentException.class)
+    public void testAddPreDestroyMethodConflicts() {
+        StandardContext standardContext = new StandardContext();
+        standardContext.addPreDestroyMethod("a", "a");
+        standardContext.addPreDestroyMethod("a", "b");
+    }
 }
